package com.osoboo.jira.metadata;

import java.util.Set;

import com.atlassian.activeobjects.tx.Transactional;

/**
 * Will be used to load / store / delete the metadata.
 * 
 * @author aspall
 * 
 */
@Transactional
public interface MetadataService {

	/**
	 * saves the given value for the enrichedObject / key combination.
	 * 
	 * @param enrichedObject
	 * @param key
	 * @param value
	 */
	void save(Object enrichedObject, String key, String value);

	/**
	 * returns all metadatas for the given enriched Object
	 * 
	 * @param enrichedObject
	 * @return
	 */
	Set<Metadata> getMetadata(Object enrichedObject);

	/**
	 * deletes the metadata specified by the enriched Object and the unique id
	 * of the metadata.
	 * 
	 * @param enrichedObject
	 * @param id
	 */
	void delete(Object enrichedObject, int id);

	/**
	 * returns the metadata value for the given enrichedObject / key
	 * combination.
	 * 
	 * @param enrichedObject
	 * @param key
	 * @return
	 */
	String getMetadataValue(Object enrichedObject, String key);

}
