package com.osoboo.jira.metadata.internal;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import com.atlassian.activeobjects.external.ActiveObjects;
import com.atlassian.jira.project.ProjectImpl;
import com.osoboo.jira.metadata.Metadata;
import com.osoboo.jira.metadata.MetadataService;

public class MetadataServiceImpl implements MetadataService {

	private final ActiveObjects ao;
	@SuppressWarnings("rawtypes")
	private Map<Class, EnrichedObjectKeyGenerator> registeredGenerators = new HashMap<Class, EnrichedObjectKeyGenerator>();

	public MetadataServiceImpl(ActiveObjects ao) {
		this.ao = ao;
		registeredGenerators.put(ProjectImpl.class, new KeyGeneratorProject());
	}

	/** {@inheritDoc} */
	@Override
	public void save(Object enrichedObject, String key, String value) {
		String enrichedObjectKey = getEnrichedObjectKey(enrichedObject);
		Metadata metadata = null;
		Metadata[] result = ao.find(Metadata.class,
				"enriched_object_key = ? AND key = ?", enrichedObjectKey, key);
		if (result.length > 0) {
			metadata = result[0];
		} else {
			metadata = ao.create(Metadata.class);
			metadata.setEnrichedObjectKey(enrichedObjectKey);
			metadata.setKey(key);
		}
		metadata.setValue(value);
		metadata.save();
	}

	/** {@inheritDoc} */
	@Override
	public String getMetadataValue(Object enrichedObject, String key) {
		Metadata metadata = getMetadata(enrichedObject, key);
		if (metadata == null) {
			return "";
		}
		return metadata.getValue();
	}

	/** {@inheritDoc} */
	@Override
	public Set<Metadata> getMetadata(Object enrichedObject) {
		String enrichedObjectKey = getEnrichedObjectKey(enrichedObject);
		try {
			Metadata[] result = ao.find(Metadata.class,
					"enriched_object_key = ?", enrichedObjectKey);
			Set<Metadata> resultSet = new HashSet<Metadata>();
			for (Metadata metadata : result) {
				resultSet.add(metadata);
			}
			return resultSet;
		} catch (Exception e) {
			e.printStackTrace();
			return new HashSet<Metadata>();
		}
	}

	/** {@inheritDoc} */
	@Override
	public void delete(Object enrichedObject, int id) {
		String enrichedObjectKey = getEnrichedObjectKey(enrichedObject);
		Metadata[] result = ao.find(Metadata.class,
				"enriched_object_key = ? AND id = ?", enrichedObjectKey, id);
		ao.delete(result);
	}

	/**
	 * @param enrichedObject
	 *            the enriced object
	 * @param key
	 *            the metadata key
	 * @return the string value
	 */
	private Metadata getMetadata(Object enrichedObject, String key) {
		String enrichedObjectKey = getEnrichedObjectKey(enrichedObject);
		Metadata[] result = ao.find(Metadata.class,
				"enriched_object_key = ? AND key = ?", enrichedObjectKey, key);
		if (result.length > 0) {
			return result[0];
		}
		return null;
	}

	/**
	 * @param enrichedObject
	 * @return the generated object key will be returned
	 */
	private String getEnrichedObjectKey(Object enrichedObject) {
		if (enrichedObject != null
				&& registeredGenerators.containsKey(enrichedObject.getClass())) {
			return registeredGenerators.get(enrichedObject.getClass()).getKey(
					enrichedObject);
		}
		return null;
	}

}
