package com.osoboo.jira.metadata.ui.field;

import java.util.List;

import com.atlassian.core.util.collection.EasyList;
import com.atlassian.jira.issue.Issue;
import com.atlassian.jira.issue.customfields.config.item.DefaultValueConfigItem;
import com.atlassian.jira.issue.customfields.impl.CalculatedCFType;
import com.atlassian.jira.issue.customfields.impl.FieldValidationException;
import com.atlassian.jira.issue.customfields.manager.GenericConfigManager;
import com.atlassian.jira.issue.customfields.persistence.CustomFieldValuePersister;
import com.atlassian.jira.issue.customfields.persistence.PersistenceFieldType;
import com.atlassian.jira.issue.fields.CustomField;
import com.atlassian.jira.issue.fields.config.FieldConfig;
import com.atlassian.jira.issue.fields.config.FieldConfigItemType;
import com.atlassian.jira.util.JiraUtils;
import com.atlassian.jira.util.collect.CollectionBuilder;

@SuppressWarnings("unchecked")
public abstract class MetadataCalculatedCFType extends CalculatedCFType {

	private CustomFieldValuePersister customFieldValuePersister;
	private GenericConfigManager genericConfigManager;

	protected MetadataCalculatedCFType(
			CustomFieldValuePersister customFieldValuePersister,
			GenericConfigManager genericConfigManager) {
		this.customFieldValuePersister = customFieldValuePersister;
		this.genericConfigManager = genericConfigManager;
	}

	@Override
	public Object getSingularObjectFromString(String arg0)
			throws FieldValidationException {
		return arg0;
	}

	@Override
	public String getStringFromSingularObject(Object arg0) {
		// TODO Auto-generated method stub
		if (arg0 == null) {
			return "";
		}
		return arg0.toString();
	}

	@SuppressWarnings("rawtypes")
	@Override
	public List getConfigurationItemTypes() {
		return CollectionBuilder.<FieldConfigItemType> newBuilder(
				JiraUtils.loadComponent(DefaultValueConfigItem.class))
				.asArrayList();
	}

	public void createValue(CustomField field, Issue issue, Object value) {
		customFieldValuePersister.createValues(field, issue.getId(),
				getDatabaseType(), EasyList.build(getDbValueFromObject(value)));
	}

	protected PersistenceFieldType getDatabaseType() {
		return PersistenceFieldType.TYPE_LIMITED_TEXT;
	}

	/**
	 * the value does exist, and the new value is different than the existing
	 * one.
	 */
	public void updateValue(CustomField customField, Issue issue, Object value) {
		customFieldValuePersister.updateValues(customField, issue.getId(),
				getDatabaseType(), EasyList.build(getDbValueFromObject(value)));
	}

	private Object getDbValueFromObject(Object value) {
		return value;
	}

	public void setDefaultValue(FieldConfig fieldConfig, Object value) {
		genericConfigManager.update(DEFAULT_VALUE_TYPE, fieldConfig.getId()
				.toString(), getDbValueFromObject(value));
	}

	public Object getDefaultValue(FieldConfig fieldConfig) {
		try {
			Object databaseValue = genericConfigManager.retrieve(
					DEFAULT_VALUE_TYPE, fieldConfig.getId().toString());
			if (databaseValue != null) {
				return getObjectFromDbValue(databaseValue);
			} else {
				return null;
			}
		} catch (FieldValidationException e) {
			return null;
		}
	}

	private Object getObjectFromDbValue(Object databaseValue) {
		return databaseValue;
	}

}
